locals {
  engine                    = "aurora-postgresql"
  db_cluster_name           = var.db_cluster_name
  final_snapshot_identifier = "${local.db_cluster_name}-final-snapshot"
  storage_encrypted         = true
  #TODO: should probably add these as input contracts as these values might change for production
  skip_final_snapshot = true

  ## aws_rds_cluster_role_association
  rds_cluster_role_association_enabled = var.rds_cluster_role_association != null ? true : false
}

resource "aws_rds_cluster" "aurora_postgresql" {
  apply_immediately         = var.apply_immediately
  cluster_identifier        = local.db_cluster_name
  engine                    = local.engine
  engine_version            = var.engine_version
  database_name             = var.database_name
  master_username           = var.admin_username
  master_password           = var.password
  backup_retention_period   = var.backup_retention_period
  preferred_backup_window   = var.backup_window
  storage_encrypted         = local.storage_encrypted
  skip_final_snapshot       = local.skip_final_snapshot
  final_snapshot_identifier = local.final_snapshot_identifier
  deletion_protection       = var.deletion_protection
  db_subnet_group_name      = aws_db_subnet_group.aurora_subnet_group.name
  vpc_security_group_ids    = var.security_group_ids
  iam_roles                 = var.aurora_cluster_role_arns
  port                      = var.port

  db_cluster_parameter_group_name = var.cluster_parameter_group_name != null ? var.cluster_parameter_group_name : local.enable_custom_aws_rds_cluster_parameter_group ? join("", aws_rds_cluster_parameter_group.aurora_cluster_parameter_group[*].id) : null

  allow_major_version_upgrade      = var.allow_major_version_upgrade
  db_instance_parameter_group_name = var.allow_major_version_upgrade ? var.instance_parameter_group_name != null ? var.instance_parameter_group_name : local.enable_custom_aws_db_parameter_group ? join("", aws_db_parameter_group.aurora_db_parameter_group[*].id) : null : null

  # this line is used for initial deploy, removed after promotion and redeployed with the enablement of the replica instances.
  replication_source_identifier = var.enable_replicas ? "" : var.replication_source_identifier
  tags                          = var.tags
}

resource "aws_rds_cluster_instance" "instances" {
  count                        = var.enable_failover ? var.replica_count + 1 : 1
  apply_immediately            = var.apply_immediately
  promotion_tier               = 0
  identifier                   = "${local.db_cluster_name}-${format("%02d", count.index + 1)}"
  cluster_identifier           = aws_rds_cluster.aurora_postgresql.id
  instance_class               = var.instance_type
  auto_minor_version_upgrade   = false # See Chart.yaml for notes on why this is set to false
  engine                       = aws_rds_cluster.aurora_postgresql.engine
  db_parameter_group_name      = var.instance_parameter_group_name != null ? var.instance_parameter_group_name : local.enable_custom_aws_db_parameter_group ? join("", aws_db_parameter_group.aurora_db_parameter_group[*].id) : null
  performance_insights_enabled = var.enable_performance_insights
  tags                         = var.tags
}

resource "aws_rds_cluster_role_association" "default" {
  count = local.rds_cluster_role_association_enabled ? 1 : 0

  db_cluster_identifier = aws_rds_cluster.aurora_postgresql.id
  feature_name          = var.rds_cluster_role_association.featureName
  role_arn              = var.rds_cluster_role_association.roleArn
}